/**
 * Candy Plugin for WebRTC Conferencing
 *
 */
 
var CandyShop = (function(self) { return self; }(CandyShop || {}));

CandyShop.WebRTC = (function(self, Candy, $) {

	var oldRoom = null;
	var oldJid = null;
	var voiceJid = {};	
	var newJid = null;	
	var newRoom = null;	
	var myNick = null;	


	self.init = function(url)
	{
		Candy.View.Event.Room.onAdd = handleOnAdd;
		Candy.View.Event.Room.onShow = handleOnShow;
		Candy.View.Event.Room.onHide = handleOnHide;
		Candy.View.Event.Room.onClose = handleOnClose;	
		Candy.View.Event.Room.onPresenceChange = handleOnPresenceChange;		
		Candy.View.Event.Roster.onUpdate = handleRoster;
				
		Candy.Core.Event.addObserver(Candy.Core.Event.KEYS.CHAT, chatObserver);
		
		WebRtc.log = Candy.Core.log;		
		WebRtc.callback = handleMuteSignal;	
		WebRtc.init(Candy.Core.getConnection(), {"iceServers": [{"url": "stun:stun.l.google.com:19302"}]});		
	};


	var handleMuteSignal = function(peer, type, muc, audioMuted, videoMuted)
	{
		Candy.Core.log("handleMuteSignal " + peer.farParty + " " + audioMuted + " " + videoMuted + " " + type + " " + muc);

		var name = peer.farParty.split("@")[0];
		var jid = peer.farParty;
		
		if (muc)
		{
			name = Strophe.unescapeNode(peer.farParty.split("/")[1]);
			
			if (type == "room")
			{
				jid = peer.farParty.split("/")[0]
			}
		}		

		var message = (!audioMuted ? "Started" : "Stopped") + " sending audio";
		
		Candy.View.Pane.Message.show(jid, name, message);	
	
	};
	
	var handleOutgoingPackets = function(msg) {
		
		Candy.Core.log('SENT: ' + msg);
		
		if (msg.indexOf("jabber:iq:privacy") > -1)
		{
			var packet = WebRtc.textToXML(msg);

			if (packet.getElementsByTagName("item").length > 0)
			{
				var elem = packet.getElementsByTagName("item")[0];
				
				var userJid = elem.getAttribute("value");
				var action =  elem.getAttribute("action");
				
				Candy.Core.log("handleOutgoingPackets found privacy user " + userJid + " " + action);	
				
				if (action == "deny" && userJid != null)
				{
					WebRtc.muteRemoteUser(true, userJid);
				}
				
				if (action == "allow" && userJid != null)
				{
					WebRtc.muteRemoteUser(false, userJid);
				}				
			}
		}
						
            	return true;		
	};
	
	var handleRoster = function(args) {
	
		Candy.Core.log("handleRoster " + args.action + " " + args.user.getEscapedJid() + " " + args.roomJid);

		var room = Strophe.getNodeFromJid(args.roomJid)
		var myJid = args.roomJid + "/" + myNick;
		
		WebRtc.handleRoster(myJid, args.user.getEscapedJid(), room, args.action)
						
            	return true;		
	};
	
	var _webrtcCallback = function(message) {

		var room = Strophe.getNodeFromJid(message.getAttribute("from"));
		var jid = message.getAttribute("from");
	
		Candy.Core.log("_webrtcCallback " + jid);
		Candy.Core.log(message);
		
		WebRtc.handleMessage(message, jid, room);
		
            	return true;		
	};


	var _jingleNodesCallback = function(iq) {

		var jid = iq.getAttribute("id");
		
		Candy.Core.log("_jingleNodesCallback " + jid);
		Candy.Core.log(iq);
		
		WebRtc.handleMessage(iq, jid, null);
		
            	return true;		
	}	

	var chatObserver = 
	{
		update: function(obj, args) 
		{
			if(args.type === 'connection')
			{
				switch(args.status) 
				{
					case Strophe.Status.CONNECTING:
						Candy.Core.addHandler(_webrtcCallback, "http://webrtc.org/xmpp", 'message');
						Candy.Core.addHandler(_jingleNodesCallback, "http://jabber.org/protocol/jinglenodes#channel", 'iq', 'result');						
						
						Candy.Core.getConnection().rawOutput = handleOutgoingPackets;
						break;
						
					case Strophe.Status.CONNECTED:	
						myNick = Strophe.escapeNode(Candy.Core.getUser().getNick());
						break;
						
				}
			}
		}
	};
			
	
	
	var handleOnAdd = function(arg) {

		voiceJid[arg.roomJid] = false;		
		Candy.Core.log("handleOnAdd " + arg.roomJid);		
	
	};	

	var handleOnShow = function(arg) 
	{
		oldJid = newJid;
		newJid = arg.roomJid;
		
		oldRoom = newRoom;		
		newRoom = newJid.split('@')[0];
		
		var privateChatName = newJid.indexOf('/') > -1 ? newJid.split('/')[1] : null;	

		Candy.Core.log("handleOnShow old " + oldJid + " new " + newJid + " private " + privateChatName);

		if (voiceJid[oldJid] == true)
		{		
			if (oldJid.indexOf('/') > -1 && Candy.View.Pane.Chat.rooms[newJid].usercount > 1)
			{				
				Candy.View.Pane.Message.show(oldJid, Strophe.unescapeNode(myNick), "Stopped sending audio");
				WebRtc.muteUser(true, escapeUserJid(oldJid));
			}

			voiceJid[oldJid] = false;

			if (oldJid.indexOf('/') == -1 && Candy.View.Pane.Chat.rooms[newJid].usercount > 1)
			{					
				Candy.View.Pane.Message.show(oldJid, Strophe.unescapeNode(myNick), "Stopped sending audio");
				WebRtc.muteRoom(true, oldRoom);
			}	


		} else if (oldJid == newJid)	{ // only happen when tab is clicked again, once	

			if (privateChatName != null && Candy.View.Pane.Chat.rooms[newJid].usercount > 1)
			{
				Candy.View.Pane.Message.show(newJid, Strophe.unescapeNode(myNick), "Started sending audio");
				WebRtc.muteUser(false, escapeUserJid(newJid));
			}				   


			if (privateChatName == null && Candy.View.Pane.Chat.rooms[newJid].usercount > 1)
			{					
				Candy.View.Pane.Message.show(newJid, Strophe.unescapeNode(myNick), "Started sending audio");				
				WebRtc.muteRoom(false, newRoom);
			}

			voiceJid[newJid] = true;				

		}			
	};
	
	var handleOnHide = function(arg) {
	
		Candy.Core.log("handleOnHide " + arg.roomJid);			

	};	

	var handleOnClose = function(arg) {
		
		Candy.Core.log("handleOnClose " + arg.roomJid);
				
		if (arg.roomJid.indexOf("/") == -1)		// don't close room, it is a private chat
		{			
			var myJid = arg.roomJid + "/" + myNick;		
			var room = arg.roomJid.split('@')[0];
			WebRtc.handleRoster(myJid, myJid, room, "leave")			
		}
		


	};
	
	var handleOnPresenceChange = function(arg) {
		
		Candy.Core.log("handleOnPresenceChange " + arg.roomJid);

	};
	
	var escapeUserJid = function(jid)
	{
		var _node = Strophe.getNodeFromJid(jid);
		var _domain = Strophe.getDomainFromJid(jid);
		var _resource = Strophe.getResourceFromJid(jid);				
		
		return _node + '@' + _domain + "/" + Strophe.escapeNode(_resource);	
	};
	
	
	return self;
}(CandyShop.WebRTC || {}, Candy, jQuery));