/*===========================================================================*
*                       		                                     *
*                                                                            *
*      Class methods     					 	     *
*                                                                            *
*                                                                            *
*============================================================================*/


WebRtc = function(room) 
{
	this.room = room;
	this.pc = null;	
	this.farParty = null;
	this.inviter = false;
	this.closed = true;
}


WebRtc.peers = {}; 
WebRtc.rooms = {}; 
WebRtc.log = function (msg) {console.log(msg)}; 


WebRtc.init =  function(connection, peerConfig)
{
	WebRtc.log("WebRtc.init");

	if (!window.webkitRTCPeerConnection) 
	{
		var msg = "webkitRTCPeerConnection not supported by this browser";			
		alert(msg);
		throw Error(msg);
	}

	WebRtc.peerConfig = peerConfig;
	WebRtc.connection = connection;
	
	navigator.webkitGetUserMedia({audio:true, video:false}, WebRtc.onUserMediaSuccess, WebRtc.onUserMediaError);
}

WebRtc.close = function()
{
	var peers = Object.getOwnPropertyNames(WebRtc.peers)
	
	for (var i=0; i< peers.length; i++)
	{
		var peer = WebRtc.peers[peers[i]];

		if (peer && peer.pc)
		{		
			peer.terminate();
			WebRtc.peers[peers[i]] = null;
			peer = null;
		}
	}
};

WebRtc.onUserMediaSuccess = function(stream)
{
	var url = webkitURL.createObjectURL(stream);
	WebRtc.localStream = stream;
};

WebRtc.onUserMediaError = function (error)
{
	WebRtc.log("WebRtc.onUserMediaError " + error.code);
};

WebRtc.setPeer = function (uniqueKey, room)
{
	WebRtc.log("WebRtc.setPeer " + uniqueKey + " " + room);
		
	if (WebRtc.peers[uniqueKey] == null)
	{
		WebRtc.peers[uniqueKey] = new WebRtc(room);
	} else {
		
		WebRtc.peers[uniqueKey].room = room;
		WebRtc.peers[uniqueKey].open();	
	}
};

WebRtc.getPeer = function (jid)
{
	WebRtc.log("WebRtc.getPeer " + jid);
	
	return WebRtc.peers[WebRtc.escape(jid)];

};


WebRtc.handleMessage = function(message, jid, room)
{
	var uniqueKey = WebRtc.escape(jid)
	
	if (message.getAttribute("type") == "error")
	{
		return;
	}
	
	if (message.getElementsByTagName("answer").length > 0)
	{
		var peer = WebRtc.peers[uniqueKey]

		if (peer && peer.pc)
		{
			peer.handleAnswer(message);

		}
		return;
	}
	
	if (message.getElementsByTagName("candidate").length > 0)
	{
		var peer = WebRtc.peers[uniqueKey]

		if (peer && peer.pc)
		{
			peer.handleCandidate(message);

		}
		return;	
	}

	if (message.getElementsByTagName("mute").length > 0)
	{
		var peer = WebRtc.peers[uniqueKey]

		if (peer && peer.pc)
		{
			peer.handleMute(message);

		}
		return;
	}
	
	if (message.getElementsByTagName("offer").length > 0)
	{
		WebRtc.setPeer(uniqueKey, room);
		WebRtc.peers[uniqueKey].handleOffer(message, jid);
		
		return;
	}

	
	var channels = message.getElementsByTagName("channel");

	if (channels.length > 0)
	{
		var peer = WebRtc.peers[uniqueKey]

		if (peer && peer.pc)
		{
			peer.handleChannel(channels[0]);

		}
		return;	
	}
};


WebRtc.handleRoster = function(myJid, jid, room, action)
{
	WebRtc.log("WebRtc.handleRoster " + myJid + " " + jid + " " + room + " " + action);

	var uniqueKey = WebRtc.escape(jid)
	
	if (action == "chat")
	{
		WebRtc.log("WebRtc.handleRoster opening chat with " + room);					
		WebRtc.rooms[room] = {ready: true, active: false};
		WebRtc.setPeer(uniqueKey, room);
		WebRtc.peers[uniqueKey].muc = false;
		WebRtc.peers[uniqueKey].initiate(jid);
			
	}
	
	if (action == "join")
	{
		if (myJid == jid)
		{
			WebRtc.log("WebRtc.handleRoster opening room " + room);					
			WebRtc.rooms[room] = {ready: true, active: false};
		}

		if (WebRtc.rooms[room] == null && myJid != jid)
		{
			WebRtc.setPeer(uniqueKey, room);
			WebRtc.peers[uniqueKey].muc = true;			
			WebRtc.peers[uniqueKey].initiate(jid);
		}			
	}	

	if (action == "leave")
	{
		if (myJid == jid)	// I have left, close all peerconnections
		{
			WebRtc.log("WebRtc.handleRoster closing room " + room);					
			WebRtc.rooms[room] = null;
			
			var peers = Object.getOwnPropertyNames(WebRtc.peers)

			for (var i=0; i< peers.length; i++)
			{
				var peer = WebRtc.peers[peers[i]];
				
				if (peer.room == room)
				{
					peer.close();	
				}
			}
			
		} else {				// someone has left, close their peerconnection
			var peer = WebRtc.peers[uniqueKey]

			if (peer != null)
			{		
				peer.close();		
			}		
		}
	}

};

WebRtc.muteRoom = function(mute, room)
{
	var peers = Object.getOwnPropertyNames(WebRtc.peers)
	
	for (var i=0; i< peers.length; i++)
	{
		WebRtc.log("Found participant " + peers[i]);		
	
		var peer = WebRtc.peers[peers[i]];

		if (peer && !peer.closed && peer.pc && peer.pc.localStreams.length > 0)
		{		
			if (peer.room == room)
			{
				WebRtc.log("muting to participant " + peer.farParty + " in room " + room + " " + mute);

				peer.pc.localStreams[0].audioTracks[0].enabled = !mute;	
				
				peer.sendMuteSignal(mute, false, "room");				
			}
		}
	}
	
	if (WebRtc.rooms[room]) WebRtc.rooms[room].active = !mute;
};

WebRtc.muteRemoteRoom = function(mute, room)
{
	var peers = Object.getOwnPropertyNames(WebRtc.peers)
	
	for (var i=0; i< peers.length; i++)
	{
		var peer = WebRtc.peers[peers[i]];

		if (peer && !peer.closed && peer.pc && peer.pc.remoteStreams.length > 0)
		{							
			if (peer.room == room)
			{
				WebRtc.log("muting from participant " + peer.farParty + " in room " + room + " " + mute);

				peer.pc.remoteStreams[0].audioTracks[0].enabled = !mute;					
			}
		}
	}
	
	if (WebRtc.rooms[room]) WebRtc.rooms[room].active = !mute;
};

WebRtc.muteUser = function(mute, jid)
{	
	var uniqueKey = WebRtc.escape(jid)

	var peer = WebRtc.peers[uniqueKey]

	if (peer != null && peer.pc && peer.pc.localStreams.length > 0)
	{		
		WebRtc.log("muting local user " + peer.farParty + " " + mute);

		peer.pc.localStreams[0].audioTracks[0].enabled = !mute;	
		
		peer.sendMuteSignal(mute, false, "private");		
	}	
};

WebRtc.muteRemoteUser = function(mute, jid)
{
	var uniqueKey = WebRtc.escape(jid)

	var peer = WebRtc.peers[uniqueKey]

	if (peer != null && peer.pc && peer.pc.remoteStreams.length > 0)
	{		
		WebRtc.log("muting remote user " + peer.farParty + " " + mute);

		peer.pc.remoteStreams[0].audioTracks[0].enabled = !mute;			
	}
};

WebRtc.textToXML = function(text)
{
	var doc = null;

	if (window['DOMParser']) {
	    var parser = new DOMParser();
	    doc = parser.parseFromString(text, 'text/xml');

	} else if (window['ActiveXObject']) {
	    var doc = new ActiveXObject("MSXML2.DOMDocument");
	    doc.async = false;
	    doc.loadXML(text);

	} else {
	    throw Error('No DOMParser object found.');
	}

	return doc.firstChild;
};

WebRtc.escape = function(s)
{
        return s.replace(/^\s+|\s+$/g, '')
            .replace(/\\/g,  "")
            .replace(/ /g,   "")
            .replace(/\"/g,  "")
            .replace(/\&/g,  "")
            .replace(/\'/g,  "")
            .replace(/\//g,  "")
            .replace(/:/g,   "")
            .replace(/</g,   "")
            .replace(/>/g,   "")
            .replace(/\./g,  "")            
            .replace(/@/g,   "");

};

/*===========================================================================*
*                       		                                     *
*                                                                            *
*                Object Methods						     *
*                                                                            *
*                                                                            *
*============================================================================*/



WebRtc.prototype.initiate = function(farParty)
{
	WebRtc.log("initiate " + farParty);

	this.farParty = farParty;
	this.inviter = true;	
	this.createPeerConnection();
	
	if (this.pc != null)
	{
		var _webrtc = this;
		
		this.pc.createOffer( function(desc) 
		{
			_webrtc.pc.setLocalDescription(desc);
			_webrtc.sendSDP(desc.sdp); 				

		});		
	}
}

WebRtc.prototype.terminate = function ()
{
	WebRtc.log("terminate");

	if (this.pc != null) this.pc.close();
	this.pc = null;	
}

WebRtc.prototype.open = function ()
{
	WebRtc.log("open");

	if (this.pc)
	{
		if (this.pc.remoteStreams.length > 0)
		{
			this.pc.remoteStreams[0].audioTracks[0].enabled = true;	
		}
		if (this.pc.localStreams.length > 0)
		{
			this.pc.localStreams[0].audioTracks[0].enabled = false;	
		}
		
		this.closed = false;		
	}
}

WebRtc.prototype.close = function ()
{
	WebRtc.log("close");

	if (this.pc)
	{
		if (this.pc.remoteStreams.length > 0)
		{
			this.pc.remoteStreams[0].audioTracks[0].enabled = false;	
		}
		if (this.pc.localStreams.length > 0)
		{
			this.pc.localStreams[0].audioTracks[0].enabled = false;	
		}
		
		this.closed = true;		
	}
}
WebRtc.prototype.handleAnswer = function(elem)
{
	WebRtc.log("handleAnswer");
	
	var sdp = elem.getElementsByTagName("answer")[0].firstChild.data;
	this.inviter= true;
	this.pc.setRemoteDescription(new RTCSessionDescription({type: "answer", sdp : sdp}));
	
	this.addJingleNodesCandidates();	

}

WebRtc.prototype.handleOffer = function(elem, jid)
{
	WebRtc.log("handleOffer");
	
	this.createPeerConnection();
	
	var sdp = elem.getElementsByTagName("offer")[0].firstChild.data;
	this.inviter= false;	
	this.farParty = jid;
	this.muc = elem.getElementsByTagName("muc").length > 0;
	this.pc.setRemoteDescription(new RTCSessionDescription({type: "offer", sdp : sdp}));

}

WebRtc.prototype.setAudioTrack = function()
{
	WebRtc.log("setAudioTrack");
	
	this.pc.localStreams[0].audioTracks[0].enabled = false;	
	
	var room = this.farParty.split("@")[0];
	
	if (WebRtc.rooms[room] && WebRtc.rooms[room].active) 
	{
		this.pc.localStreams[0].audioTracks[0].enabled = true;		
	}
}

WebRtc.prototype.handleMute = function(elem)
{
	WebRtc.log("handleMute");
	
	var mute = elem.getElementsByTagName("mute")[0];
	var audio = mute.getAttribute("audio");
	var video = mute.getAttribute("video");	
	var muc = mute.getAttribute("muc");		
	var type = mute.getAttribute("type");
	
	if (WebRtc.callback) WebRtc.callback(this, type, muc == "true", audio == "true", video == "true");
}

WebRtc.prototype.handleCandidate = function(elem)
{
	WebRtc.log("handleCandidate");
	
	var candidate = elem.getElementsByTagName("candidate")[0];
	var ice = {sdpMLineIndex: candidate.getAttribute("label"), candidate: candidate.getAttribute("candidate")};				
	this.pc.addIceCandidate(new RTCIceCandidate(ice));	
}

WebRtc.prototype.handleChannel = function(channel)
{
	WebRtc.log("handleChannel");

	var relayHost = channel.getAttribute("host");
	var relayLocalPort = channel.getAttribute("localport");
	var relayRemotePort = channel.getAttribute("remoteport");

	WebRtc.log("add JingleNodes candidate: " + relayHost + " " + relayLocalPort + " " + relayRemotePort); 

	this.sendTransportInfo("0", "a=candidate:3707591233 1 udp 2113937151 " + relayHost + " " + relayRemotePort + " typ host generation 0");				

	var candidate = new RTCIceCandidate({sdpMLineIndex: "0", candidate: "a=candidate:3707591233 1 udp 2113937151 " + relayHost + " " + relayLocalPort + " typ host generation 0"});				
	this.pc.addIceCandidate(candidate);				
}
	

WebRtc.prototype.createPeerConnection = function()
{
	WebRtc.log("createPeerConnection");

	this.pc = new window.webkitRTCPeerConnection(WebRtc.peerConfig);

	this.pc.onicecandidate = this.onIceCandidate.bind(this);		
	this.pc.onstatechange = this.onStateChanged.bind(this);
	this.pc.onopen = this.onSessionOpened.bind(this);
	this.pc.onaddstream = this.onRemoteStreamAdded.bind(this);
	this.pc.onremovestream = this.onRemoteStreamRemoved.bind(this);
	
	this.pc.addStream(WebRtc.localStream);
	
	this.closed = false;	
}

WebRtc.prototype.onIceCandidate = function (event)
{
	WebRtc.log("onIceCandidate");
	
	if (event.candidate && this.closed == false)
	{		
		this.sendTransportInfo(event.candidate.sdpMLineIndex, event.candidate.candidate);
	}	
		
}


WebRtc.prototype.onSessionOpened = function (event)
{
	WebRtc.log("onSessionOpened");
	WebRtc.log(event);
}

WebRtc.prototype.onRemoteStreamAdded = function (event)
{
	var url = webkitURL.createObjectURL(event.stream);
	WebRtc.log("onRemoteStreamAdded " + url);
	WebRtc.log(event);
	
	if (this.inviter == false)
	{
	    var _webrtc = this;	
	    
	    this.pc.createAnswer( function (desc)
	    {
		_webrtc.pc.setLocalDescription(desc);			
		_webrtc.sendSDP(desc.sdp); 	
		
	    });			
	}
	
	this.setAudioTrack();	

	   var uniqueId = "webrtc_" + WebRtc.escape(this.farParty);
	   
	   if (!document.getElementById(uniqueId))	   
	   {
	   	var ifrm = document.createElement("video"); 
	   	ifrm.setAttribute("id", uniqueId); 	   
		ifrm.setAttribute("autoplay", "autoplay"); 	   	
	   	ifrm.style.display = "none"; 
	   	document.body.appendChild(ifrm);	   
	   } 

 	   document.getElementById(uniqueId).src = url; 
}

WebRtc.prototype.onRemoteStreamRemoved = function (event)
{
	//var url = webkitURL.createObjectURL(event.stream);
	WebRtc.log("onRemoteStreamRemoved " + url);
	WebRtc.log(event);
}

WebRtc.prototype.onStateChanged = function (event)
{
	WebRtc.log("onStateChanged");
	WebRtc.log(event);
}



WebRtc.prototype.sendSDP = function(sdp)
{
	WebRtc.log("sendSDP");
	WebRtc.log(sdp);	
	
	var msg = "";
	msg += "<message  type='chat' to='" + this.farParty + "'>";	
	msg += "<webrtc xmlns='http://webrtc.org/xmpp'>";
	
	if (this.inviter)
		msg += "<offer>" + sdp + "</offer>";
	else
		msg += "<answer>" + sdp + "</answer>";

	if (this.muc) msg += "<muc/>";
	
	msg += "</webrtc>";	
	msg += "</message>";	
	
	this.sendPacket(msg);
}

	
WebRtc.prototype.sendMuteSignal = function (audio, video, type)
{
	WebRtc.log("sendMuteSignal " + audio + " " + video + " " + type);
		
	var msg = "";
	msg += "<message type='chat' to='" + this.farParty + "'>";	
	msg += "<webrtc xmlns='http://webrtc.org/xmpp'>";
	msg += "<mute audio='" + audio + "' video='" + video + "' muc='" + this.muc + "' type='" + type + "' />";	
	msg += "</webrtc>";	
	msg += "</message>";	
	
	this.sendPacket(msg);
}

WebRtc.prototype.sendTransportInfo = function (sdpMLineIndex, candidate)
{
	WebRtc.log("sendTransportInfo");
	
	var msg = "";
	msg += "<message type='chat' to='" + this.farParty + "'>";	
	msg += "<webrtc xmlns='http://webrtc.org/xmpp'>";
	msg += "<candidate label='" + sdpMLineIndex + "' candidate='" + candidate + "' />";	
	msg += "</webrtc>";	
	msg += "</message>";	
	
	this.sendPacket(msg);	
}


WebRtc.prototype.addJingleNodesCandidates = function() 
{
	WebRtc.log("addJingleNodesCandidates");
	
	var iq = "";
	var id = this.farParty;
		
	iq += "<iq type='get' to='" +  "relay." + window.location.hostname + "' id='" + id + "'>";
	iq += "<channel xmlns='http://jabber.org/protocol/jinglenodes#channel' protocol='udp' />";
	iq += "</iq>";	

	this.sendPacket(iq);	
}


WebRtc.prototype.sendPacket = function(packet) {

	try {	
		if (WebRtc.connection instanceof Strophe.Connection) 
		{	
			var xml = WebRtc.textToXML(packet);

			WebRtc.log("sendPacket with Strophe.Connection");
			WebRtc.log(xml);		

			WebRtc.connection.send(xml);		

		} else {

			WebRtc.log("sendPacket as String");
			WebRtc.log(packet);

			WebRtc.connection.sendXML(packet);
		}
	
	} catch (e) {

		WebRtc.log("sendPacket as String");
		WebRtc.log(packet);

		WebRtc.connection.sendXML(packet);	
	}
};

